/**
******************************************************************************
* @file    IRPyro_UT_HL_for_TO.c
* @author  _KEMET, Ltd
* @date    March 2018
* @version Release 1.0.0
* @copyright (c) 2018 _KEMET, Ltd
* @brief   High Level Unit Testing module
******************************************************************************
   @attention <h2><center>&copy; COPYRIGHT 2018 _KEMET, Ltd</center></h2>

   @verbatim
    Copyright (c) 2018, _KEMET, Ltd
    All rights reserved.

    THIS SOFTWARE IS PROVIDED BY _KEMET, Ltd ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL _KEMET, Ltd BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
   @endverbatim
******************************************************************************
*/
/* Includes ------------------------------------------------------------------*/
#include "IRPyro_UT.h"
void TO_test01(void);
void TO_test02(void);
void TO_test03(void);
void TO_test04(void);
void TO_test05(void);
void TO_test06(void);
void TO_test07(void);
static void sensor_power_stabilize(void);
/** @defgroup IRPyroAPI_UnitTest_HL High level unit test module
 * @brief Configure the IRPyro sensors to test and calls the functions defined in IRPyroUT.c
 * @{
 */
/** @defgroup UT_01_IRPyro Main function
 * @brief Calls the product verification plan units
 *        Each function validates a point of the product verification plan
 * @note Refer to PSVP017 (product verification specification) document on the engineering section.
 * @{
 */
/**
 * @brief unit testing main program.
  */
int main()
{
    /*-------------------------- board setup ---------------------------------*/
    Micro_controller_template_init();
    sensor_power_stabilize();
    // turn off all CS lines
    IRPyro_demo_board(0);
    /*-------------------------- start test ----------------------------------*/
    /*Single Sensor Test */
//    TO_test01(); // 01 Initialize a single device for POLLED CONFIGURATION
//    TO_test03(); // 03 READ DATA FROM A SINGLE SENSOR and detect frames missing with the sensor CONFIGURED FOR POLLED
//    TO_test05(); // 05 Change sensor POWER CONFIGURATION
//    TO_test06(); // 06 Configure sensor WAKE UP EVENT, SEND TO SLEEP, DETECT WAKE UP
//    TO_test07(); // 07 Change PARAMETERS on the fly

    /*Multiple Sensor Test */
//    TO_test02(); // 02 Initialize multiple devices for POLLED CONFIGURATION
    TO_test04(); // 04 READ DATA FROM MULTIPLE SENSORS and detect frames missing with the sensors CONFIGURED FOR POLLED
}
/** end of main program
 * @}
 */
/** @defgroup ver_plan Verification plan functions
 * @brief each function configures the required data structures to pass to the
 *        unit test functions
 * @{
 */
/**
 * @brief Configures the registers of a single IRPyro TO and then read back again.
 * @attention Set your serial terminal console with the following parameters:
 * @verbatim
 *        Baud Rate:           921600
 *        Parity:                None
 *        Data Bits:                8
 *        Stop Bits:                1
 *        Hardware Flow Control: None
 *        Software Flow Control: None
 * @endverbatim
 *        The function sends the result of the operation to the serial console
 */
void TO_test01(void)
{
    /*------------------ Memory allocation, initialization-------------------*/
    IRPyro_sensor_device_type IRPyro_single_sensor = {0}; // Instance of sensor
    /*-------------------Physical Location ----------------------------------*/
    IRPyro_single_sensor.INT_Port = 1;
    IRPyro_single_sensor.CS_Port  = 2;
    IRPyro_single_sensor.CS_Pin   = 1;
    IRPyro_single_sensor.INT_Pin  = 4;
    /*-------------------Test configuration ---------------------------------*/
    IRPyro_single_sensor.number_of_active_channels = 1;   // configures channel 2 for streaming
    IRPyro_single_sensor.read_mode = 1;                   // POLLED CONFIGURATION
    IRPyro_single_sensor.address   = 0x08;                // I2C address to configure
    /*-------------------------- board setup ---------------------------------*/
    sensor_power_stabilize();
    IRPyro_demo_board(0);
    UART_PutString(USART2, (uint8_t*)"01 IRPyro TO: Initialize a single device \n");
    ut01_single_sensor_init(&IRPyro_single_sensor);
}
/**
 * @brief Configures a list of IRPyro TO. The configuration data is read from the sensors and compared.
 * @attention Set your serial terminal console with the following parameters:
 * @verbatim
 *        Baud Rate:           921600
 *        Parity:                None
 *        Data Bits:                8
 *        Stop Bits:                1
 *        Hardware Flow Control: None
 *        Software Flow Control: None
 * @endverbatim
 *        The function sends the result of the operation to the serial console
 */
void TO_test02(void)
{
    /*------------------- Memory allocation, initialization ------------------*/
    IRPyro_devices IRPyro_device = {0};
    /*----------------------- --- CONFIGURATION -----  -----------------------*/
    AFE_reg_type IRPyro_AFE_configuration =
    {
        .CLK_OUT = CLK_DISABLE,
        .INT  = INT_DISABLE
    };
    /*-----------------    ADDRESSES   ------------*/
    /* Assuming four sensors on an IRPyro Break out board or demo board*/
    IRPyro_device[0].INT_Port = 1;
    IRPyro_device[0].CS_Port  = 2;
    IRPyro_device[0].CS_Pin   = 1;
    IRPyro_device[0].INT_Pin  = 4;
    IRPyro_device[0].address  = 0x70;

    IRPyro_device[1].CS_Port  = 2;
    IRPyro_device[1].INT_Port = 1;
    IRPyro_device[1].CS_Pin   = 5;
    IRPyro_device[1].INT_Pin  = 5;
    IRPyro_device[1].address  = 0x69;

    IRPyro_device[2].CS_Port  = 2;
    IRPyro_device[2].INT_Port = 1;
    IRPyro_device[2].CS_Pin   = 4;
    IRPyro_device[2].INT_Pin  = 6;
    IRPyro_device[2].address  = 0x68;

    /*-----------------    CONFIGURATION   ------------*/
    IRPyro_device[0].AFE_register    = IRPyro_AFE_configuration; // MASTER, INT, CLK DISABLED
    IRPyro_device[0].number_of_active_channels = 1;              // configures channel 2 for streaming
    IRPyro_device[0].read_mode       = 1;
    IRPyro_device[1].AFE_register    = IRPyro_AFE_configuration; // MASTER, INT, CLK DISABLED
    IRPyro_device[1].number_of_active_channels = 1;              // configures channel 2 for streaming
    IRPyro_device[1].read_mode       = 1;
    IRPyro_device[2].AFE_register    = IRPyro_AFE_configuration; // MASTER, INT, CLK DISABLED
    IRPyro_device[2].number_of_active_channels = 1;              // configures channel 2 for streaming
    IRPyro_device[2].read_mode       = 1;
    sensor_power_stabilize();
    IRPyro_demo_board(0);
    UART_PutString(USART2, (uint8_t*)"02 IRPyro TO: Initialize multiple devices \n");
    ut02_multiple_sensor_init(IRPyro_device, 3);
}
/**
 * @brief Configures a single IRPyro TO for data streaming and verifies if any frame is missing
 * One hour of data is read from the sensor. Each frame counter received is compared
 * with the previous. A missing frame is detected when both numbers are NOT CONSECUTIVE.
 * This event is counted.
 * @attention Set your serial terminal console with the following parameters:
 * @verbatim
 *        Baud Rate:           921600
 *        Parity:                None
 *        Data Bits:                8
 *        Stop Bits:                1
 *        Hardware Flow Control: None
 *        Software Flow Control: None
 * @endverbatim
 *        The function sends the result of the operation to the serial console
 */
void TO_test03(void)
{
    /*------------------ Memory allocation, initialization-------------------*/
    IRPyro_sensor_device_type IRPyro_single_sensor = {0}; // Instance of sensor
    /*-------------------Physical Location ----------------------------------*/
    IRPyro_single_sensor.INT_Port = 1;
    IRPyro_single_sensor.CS_Port  = 2;
    IRPyro_single_sensor.CS_Pin   = 1;
    IRPyro_single_sensor.INT_Pin  = 4;
    /*-------------------Test configuration ---------------------------------*/
    IRPyro_single_sensor.number_of_active_channels = 1;   // configures channel 2 for streaming
    IRPyro_single_sensor.read_mode = 1;                   // Polled mode reading
    IRPyro_single_sensor.address   = 0x08;                // I2C address to configure
    /*-------------------------- board setup ---------------------------------*/
    sensor_power_stabilize();
    IRPyro_demo_board(0);
    UART_PutString(USART2, (uint8_t*)"03 IRPyro TO: READ DATA FROM A SINGLE IRPyro TO SENSOR and detect frames missing\n");
    ut03_single_sensor_read(&IRPyro_single_sensor);
}
/**
 * @brief A maximum of three IRPyro TO sensors are configured for data read with frame counter detection
configured. One hour of data is read from the sensors. Each frame counter received
is compared with the previous. A missing frame is detected when both numbers are
NOT CONSECUTIVE. This event is counted.
 * @attention Set your serial terminal console with the following parameters:
 * @verbatim
 *        Baud Rate:           921600
 *        Parity:                None
 *        Data Bits:                8
 *        Stop Bits:                1
 *        Hardware Flow Control: None
 *        Software Flow Control: None
 * @endverbatim
 *        The function sends the result of the operation to the serial console
 */
void TO_test04(void)
{

    /*------------------- Memory allocation, initialization ------------------*/
    IRPyro_devices IRPyro_device = {0};
    /*----------------------- --- CONFIGURATION -----  -----------------------*/
    AFE_reg_type IRPyro_AFE_Master =
    {
        .S7_S0 = 0,
        .LP    = LOW_POWER_DISABLE,
        .HP    = HP_DISABLE,
        .C_LP  = LOW_PASS_22_5,
        .CLK_OUT = CLK_DISABLE,
        .SYNC = MASTER,
        .TEMP = 0,
        .INT  = INT_DISABLE
    };
    /*-----------------    ADDRESSES   ------------*/
    /* Assuming four sensors on an IRPyro Break out board or demo board*/
    IRPyro_device[0].INT_Port = 1;
    IRPyro_device[0].CS_Port  = 2;
    IRPyro_device[0].CS_Pin   = 1;
    IRPyro_device[0].INT_Pin  = 4;
    IRPyro_device[0].address  = 0x70;

    IRPyro_device[1].CS_Port  = 2;
    IRPyro_device[1].INT_Port = 1;
    IRPyro_device[1].CS_Pin   = 5;
    IRPyro_device[1].INT_Pin  = 5;
    IRPyro_device[1].address  = 0x69;

    IRPyro_device[2].CS_Port  = 2;
    IRPyro_device[2].INT_Port = 1;
    IRPyro_device[2].CS_Pin   = 4;
    IRPyro_device[2].INT_Pin  = 6;
    IRPyro_device[2].address  = 0x68;

    /*-----------------    CONFIGURATION   ------------*/
    IRPyro_device[0].AFE_register    = IRPyro_AFE_Master; // MASTER, INT, CLK DISABLED
    IRPyro_device[0].read_mode       = 1;
    IRPyro_device[0].number_of_active_channels = 1;

    IRPyro_device[1].AFE_register    = IRPyro_AFE_Master; // MASTER, INT, CLK DISABLED
    IRPyro_device[1].read_mode       = 1;
    IRPyro_device[1].number_of_active_channels = 1;

    IRPyro_device[2].AFE_register    = IRPyro_AFE_Master; // MASTER, INT, CLK DISABLED
    IRPyro_device[2].read_mode       = 1;
    IRPyro_device[2].number_of_active_channels = 1;

    sensor_power_stabilize();
    IRPyro_demo_board(0);
    UART_PutString(USART2, (uint8_t*)"04 IRPyro TO: READ DATA FROM MULTIPLE SENSORS and detect frames missing\n");
    ut04_multiple_sensor_read(IRPyro_device, 3);
}
/**
 * @brief Change register settings of a single IRPyro TO
The following API commands are issued: sampling_rate_set, sampling_rate_get, high_pass_set,
high_pass_get, low_pass_set, low_pass_get, gain_set, and gain_get, power_off.
A sample of readings is taken and the behaviour of the sensor is observed.
 * @attention Set your serial terminal console with the following parameters:
 * @verbatim
 *        Baud Rate:           921600
 *        Parity:                None
 *        Data Bits:                8
 *        Stop Bits:                1
 *        Hardware Flow Control: None
 *        Software Flow Control: None
 * @endverbatim
 *        The function sends the result of the operation to the serial console

 */
void TO_test05(void)
{
    /*------------------ Memory allocation, initialization-------------------*/
    IRPyro_sensor_device_type IRPyro_single_sensor = {0}; // Instance of sensor
    /*-------------------Physical Location ----------------------------------*/
    IRPyro_single_sensor.INT_Port = 1;
    IRPyro_single_sensor.CS_Port  = 2;
    IRPyro_single_sensor.CS_Pin   = 1;
    IRPyro_single_sensor.INT_Pin  = 4;
    /*-------------------Test configuration ---------------------------------*/
    IRPyro_single_sensor.read_mode = 2;                   // DAISY CHAIN CONFIGURATION
    IRPyro_single_sensor.address   = 0x08;                // I2C address to configure
    IRPyro_single_sensor.number_of_active_channels = 1;   // configures channel 2 for streaming
    /*-------------------------- board setup ---------------------------------*/
    sensor_power_stabilize();
    IRPyro_demo_board(0);
    UART_PutString(USART2, (uint8_t*)"05 IRPyro TO: Change sensor POWER CONFIGURATION\n");
    ut06_IOCTL02(&IRPyro_single_sensor);
}
/**
 * @brief Send a single IRPyro TO to sleep and then wake up by command
The following API commands are issued: power_sleep, and power_wakeup. A sample of
readings is taken and the behaviour of the sensor is observed.
 * @attention Set your serial terminal console with the following parameters:
 * @verbatim
 *        Baud Rate:           921600
 *        Parity:                None
 *        Data Bits:                8
 *        Stop Bits:                1
 *        Hardware Flow Control: None
 *        Software Flow Control: None
 * @endverbatim
 *        The function sends the result of the operation to the serial console
 */
void TO_test06(void)
{
// wake command response
    /* The sensor is configured to sleep and on a wake up event alerts the MCU and
     * data is read a more elaborate scenario is putting the MCU to sleep and the
       * sensor wakes the device up.
     */
    /*------------------ Memory allocation, initialization-------------------*/
    IRPyro_sensor_device_type IRPyro_single_sensor = {0}; // Instance of sensor
    /*-------------------Physical Location ----------------------------------*/
    IRPyro_single_sensor.INT_Port = 1;
    IRPyro_single_sensor.CS_Port  = 2;
    IRPyro_single_sensor.CS_Pin   = 1;
    IRPyro_single_sensor.INT_Pin  = 4;
    /*-------------------Test configuration ---------------------------------*/
    IRPyro_single_sensor.read_mode = 2;                   // DAISY CHAIN CONFIGURATION
    IRPyro_single_sensor.address   = 0x70;                // I2C address to configure
    IRPyro_single_sensor.number_of_active_channels = 1;   // configures channel 2 for streaming
    /*------------------- Wake up event definition  -------------------------*/
    IRPyro_single_sensor.WUP_register.UHT          = MAX_WUP_THRESHOLD;
    IRPyro_single_sensor.WUP_register.ULT          = SAFE_WUP_THRESHOLD;
    IRPyro_single_sensor.WUP_register.LHT          = MIN_WUP_THRESHOLD;
    IRPyro_single_sensor.WUP_register.LLT          = MIN_WUP_THRESHOLD;
    IRPyro_single_sensor.WUP_register.WT_Threshold = DEFAULT_NO_EVENTS;
    IRPyro_single_sensor.WUP_register.DP0_DP2      = SEL_CH_2;
    IRPyro_single_sensor.WUP_register.CH0_CH2      = SEL_CH_2;
    IRPyro_single_sensor.WUP_register.ST           = SLEEP_MODE_ONE_CH;  // Single channel selected
    /*-------------------------- board setup --------------------------------*/
    sensor_power_stabilize();
    IRPyro_demo_board(0);
    UART_PutString(USART2, (uint8_t*)"06 IRPyro TO: Configure sensor WAKE UP EVENT, SEND TO SLEEP, DETECT WAKE UP \n");
    ut_wake_up_event(&IRPyro_single_sensor);
}
/**
 * @brief Turn OFF and ON a single IRPyro TO with parameters restored
The following API commands are issued: power_off, power_normal, and power_low. A
sample of readings is taken and the behavior of the sensor is observed.
 * @attention Set your serial terminal console with the following parameters:
 * @verbatim
 *        Baud Rate:           921600
 *        Parity:                None
 *        Data Bits:                8
 *        Stop Bits:                1
 *        Hardware Flow Control: None
 *        Software Flow Control: None
 * @endverbatim
 *        The function sends the result of the operation to the serial console
 */
void TO_test07(void)
{
    /*------------------ Memory allocation, initialization-------------------*/
    IRPyro_sensor_device_type IRPyro_single_sensor = {0}; // Instance of sensor
    /*-------------------Physical Location ----------------------------------*/
    IRPyro_single_sensor.INT_Port = 1;
    IRPyro_single_sensor.CS_Port  = 2;
    IRPyro_single_sensor.CS_Pin   = 1;
    IRPyro_single_sensor.INT_Pin  = 4;
    /*-------------------Test configuration ---------------------------------*/
    IRPyro_single_sensor.read_mode = 2;                   // DAISY CHAIN CONFIGURATION
    IRPyro_single_sensor.address   = 0x08;                // I2C address to configure
    IRPyro_single_sensor.number_of_active_channels = 1;   // configures channel 2 for streaming
    /*-------------------------- board setup ---------------------------------*/
    sensor_power_stabilize();
    IRPyro_demo_board(0);
    UART_PutString(USART2, (uint8_t*)"07 IRPyro TO: Change PARAMETERS on the fly\n");
    ut06_IOCTL01(&IRPyro_single_sensor);
}
/**
 * @brief Changes the state of all the CS lines as used on the IRPyro backplane board
 * @param state   0 disabled / 1 enabled
 */
void demo_board_sensor_control(int state)
{
    CS_pin_set(2,1,state);
    CS_pin_set(2,3,state);
    CS_pin_set(2,4,state);
    CS_pin_set(2,5,state);
}
/**
 * @brief power cycles the sensors on the board to assure stability
*/
static void sensor_power_stabilize()
{
    // toggle all CS lines
    demo_board_sensor_control(0);
    Micro_controller_delay_ms(30);
    demo_board_sensor_control(1);
    Micro_controller_delay_ms(30);
    demo_board_sensor_control(0);
}
/** end of Verification plan functions
 * @}
 */
/** end of High Level Unit Test module
 * @}
 */
/* ********** Copyright (c) 2018 _KEMET, Ltd. **********END OF FILE************/
